@@ -20,7 +20,7 @@
     
     License
     -------
-    Copyright 2008 - 2009 Daniel G. Taylor <dan@programmer-art.org>
+    Copyright 2008 - 2010 Daniel G. Taylor <dan@programmer-art.org>
     
     This file is part of Arista.
 
@@ -39,12 +39,17 @@
     <http://www.gnu.org/licenses/>.
 """
 
+try:
+    import json
+except ImportError:
+    import simplejson as json
+
 import gettext
 import logging
 import os
 import sys
+import tarfile
 import urllib2
-import xml.etree.ElementTree
 
 import gobject
 import gst
@@ -56,8 +61,7 @@ _ = gettext.gettext
 _presets = {}
 _log = logging.getLogger("arista.presets")
 
-UPDATE_LOCATION = "http://programmer-art.org" + \
-                  "/media/releases/arista-transcoder/presets/"
+UPDATE_LOCATION = "http://www.transcoder.org/media/presets/"
 
 class Fraction(gst.Fraction):
     """
@@ -71,7 +75,7 @@ class Fraction(gst.Fraction):
             @param value: Either a single number or two numbers separated by
                           a '/' that represent a fraction
         """
-        parts = value.split("/")
+        parts = str(value).split("/")
         
         if len(parts) == 1:
             gst.Fraction.__init__(self, int(value), 1)
@@ -160,6 +164,126 @@ class Device(object):
             return self.model
         else:
             return "%s %s" % (self.make, self.model)
+    
+    @property
+    def default_preset(self):
+        """
+            Get the default preset for this device. If no default has been
+            defined, the first preset that was loaded is returned. If no
+            presets have been defined an exception is raised.
+            
+            @rtype: Preset
+            @return: The default preset for this device
+            @raise ValueError: No presets have been defined for this device
+        """
+        if self.default:
+            preset = self.presets[self.default]
+        elif len(self.presets):
+            preset = self.presets[0]
+        else:
+            raise ValueError(_("No presets have been defined for " \
+                                 "%(name)s") % { "name": self.name })
+        
+        return preset
+
+    @property
+    def json(self):
+        data = {
+            "make": self.make,
+            "model": self.model,
+            "description": self.description,
+            "author": {
+                "name": self.author.name,
+                "email": self.author.email,
+            },
+            "version": self.version,
+            "icon": self.icon,
+            "default": self.default,
+            "presets": [],
+        }
+
+        for name, preset in self.presets.items():
+            rates = []
+            for x in preset.acodec.rate[0], preset.acodec.rate[1], preset.vcodec.rate[0], preset.vcodec.rate[1]:
+                if isinstance(x, gst.Fraction):
+                    if x.num == x.denom:
+                        rates.append("%s" % x.num)
+                    else:
+                        rates.append("%s/%s" % (x.num, x.denom))
+                else:
+                    rates.append("%s" % x)
+        
+            data["presets"].append({
+                "name": preset.name,
+                "container": preset.container,
+                "extension": preset.extension,
+                "acodec": {
+                    "name": preset.acodec.name,
+                    "container": preset.acodec.container,
+                    "rate": [rates[0], rates[1]],
+                    "passes": preset.acodec.passes,
+                    "width": preset.acodec.width,
+                    "depth": preset.acodec.depth,
+                    "channels": preset.acodec.channels,
+                },
+                "vcodec": {
+                    "name": preset.vcodec.name,
+                    "container": preset.vcodec.container,
+                    "rate": [rates[2], rates[3]],
+                    "passes": preset.vcodec.passes,
+                    "width": preset.vcodec.width,
+                    "height": preset.vcodec.height,
+                },
+            })
+        
+        return json.dumps(data, indent=4)
+
+    @staticmethod
+    def from_json(data):
+        parsed = json.loads(data)
+
+        device = Device(**{
+            "make": parsed.get("make", "Generic"),
+            "model": parsed.get("model", ""),
+            "description": parsed.get("description", ""),
+            "author": Author(
+                name = parsed.get("author", {}).get("name", ""),
+                email = parsed.get("author", {}).get("email", ""),
+            ),
+            "version": parsed.get("version", ""),
+            "icon": parsed.get("icon", ""),
+            "default": parsed.get("default", ""),
+        })
+
+        for preset in parsed.get("presets", []):
+            acodec = preset.get("acodec", {})
+            vcodec = preset.get("vcodec", {})
+            device.presets[preset.get("name", "")] = Preset(**{
+                "name": preset.get("name", ""),
+                "container": preset.get("container", ""),
+                "extension": preset.get("extension", ""),
+                "icon": preset.get("icon", ""),
+                "acodec": AudioCodec(**{
+                    "name": acodec.get("name", ""),
+                    "container": acodec.get("container", ""),
+                    "rate": acodec.get("rate", []),
+                    "passes": acodec.get("passes", []),
+                    "width": acodec.get("width", []),
+                    "depth": acodec.get("depth", []),
+                    "channels": acodec.get("channels", []),
+                }),
+                "vcodec": VideoCodec(**{
+                    "name": vcodec.get("name", ""),
+                    "container": vcodec.get("container", ""),
+                    "rate": [Fraction(x) for x in vcodec.get("rate", [])],
+                    "passes": vcodec.get("passes", []),
+                    "width": vcodec.get("width", []),
+                    "height": vcodec.get("height", []),
+                }),
+                "device": device,
+            })
+
+        return device
 
 class Preset(object):
     """
@@ -167,7 +291,7 @@ class Preset(object):
         device.
     """
     def __init__(self, name = "", container = "", extension = "", 
-                 acodec = None, vcodec = None):
+                 acodec = None, vcodec = None, device = None, icon = None):
         """
             @type name: str
             @param name: The name of the preset, e.g. "High Quality"
@@ -179,12 +303,16 @@ class Preset(object):
             @param acodec: The audio encoding settings
             @type vcodec: VideoCodec
             @param vcodec: The video encoding settings
+            @type device: Device
+            @param device: A link back to the device this preset belongs to
         """
         self.name = name
         self.container = container
         self.extension = extension
         self.acodec = acodec
         self.vcodec = vcodec
+        self.device = device
+        self.icon = icon
     
     def __repr__(self):
         return "%s %s" % (self.name, self.container)
@@ -228,13 +356,18 @@ class Preset(object):
         ]
         
         missing = []
+        missingdesc = ""
         for element in elements:
             if not gst.element_factory_find(element):
                 missing.append(gst.pbutils.missing_element_installer_detail_new(element))
+                if missingdesc:
+                    missingdesc += ", %s" % element
+                else:
+                    missingdesc += element
         
         if missing:
+            _log.info("Attempting to install elements: %s" % missingdesc)
             if gst.pbutils.install_plugins_supported():
-                
                 def install_done(result, null):
                     if result == gst.pbutils.INSTALL_PLUGINS_INSTALL_IN_PROGRESS:
                         # Ignore start of installer message
@@ -242,12 +375,14 @@ class Preset(object):
                     elif result == gst.pbutils.INSTALL_PLUGINS_SUCCESS:
                         callback(self, True, *args)
                     else:
+                        _log.error("Unable to install required elements!")
                         callback(self, False, *args)
             
                 context = gst.pbutils.InstallPluginsContext()
                 gst.pbutils.install_plugins_async(missing, context,
                                                   install_done, "")
             else:
+                _log.error("Installing elements not supported!")
                 gobject.idle_add(callback, self, False, *args)
         else:
             gobject.idle_add(callback, self, True, *args)
@@ -257,7 +392,7 @@ class Codec(object):
         Settings for encoding audio or video. This object defines options
         common to both audio and video encoding.
     """
-    def __init__(self, name = "", container = ""):
+    def __init__(self, name=None, container=None, passes=None):
         """
             @type name: str
             @param name: The name of the encoding GStreamer element, e.g. faac
@@ -267,10 +402,11 @@ class Codec(object):
                               may not want to wrap it in an avi or mp4; if not
                               set it defaults to the preset container
         """
-        self.name = name
-        self.container = container
+        self.name = name and name or ""
+        self.container = container and container or ""
+        self.passes = passes and passes or []
+
         self.rate = (Fraction(), Fraction())
-        self.passes = []
     
     def __repr__(self):
         return "%s %s" % (self.name, self.container)
@@ -279,157 +415,22 @@ class AudioCodec(Codec):
     """
         Settings for encoding audio.
     """
-    def __init__(self, *args):
-        Codec.__init__(self, *args)
-        self.rate = (8000, 96000)
-        self.width = (8, 24)
-        self.depth = (8, 24)
-        self.channels = (1, 6)
+    def __init__(self, name=None, container=None, rate=None, passes=None, width=None, depth=None, channels=None):
+        Codec.__init__(self, name=name, container=container, passes=passes)
+        self.rate = rate and rate or (8000, 96000)
+        self.width = width and width or (8, 24)
+        self.depth = depth and depth or (8, 24)
+        self.channels = channels and channels or (1, 6)
 
 class VideoCodec(Codec):
     """
         Settings for encoding video.
     """
-    def __init__(self, *args):
-        Codec.__init__(self, *args)
-        self.rate = (Fraction("1"), Fraction("60"))
-        self.width = (2, 1920)
-        self.height = (2, 1080)
-
-def _parse_range(value, type = int):
-    """
-        Parse a string into a range.
-        
-            >>> _parse_range("1")
-            (1, 1)
-            >>> _parse_range("2, 5")
-            (2, 5)
-            >>> _parse_range("1.0, 6", type = float)
-            (1.0, 6.0)
-        
-        @type value: str
-        @param value: A string value to be parsed
-        @type type: type
-        @param type: The type to coerce value into
-    """
-    parts = value.split(",")
-    
-    if len(parts) == 1:
-        return (type(parts[0]), type(parts[0]))
-    elif len(parts) == 2:
-        return (type(parts[0]), type(parts[1]))
-    else:
-        raise ValueError(_("Value may only contain one comma; got %(value)s") % {
-            "value": value
-        })
-
-def _load_author(root):
-    """
-        Load an author from a given xml element.
-        
-        @type root: xml.etree.Element
-        @param root: An author element with name and email children
-        @rtype: Author
-        @return: A new Author instance
-    """
-    author = Author()
-    
-    for child in root.getchildren():
-        if child.tag == "name":
-            author.name = child.text.strip()
-        elif child.tag == "email":
-            author.email = child.text.strip()
-    
-    return author
-
-def _load_audio_codec(root):
-    """
-        Load an audio codec from a given xml element.
-        
-        @type root: xml.etree.Element
-        @param root: An audio codec element
-        @rtype: AudioCodec
-        @return: A new AudioCodec instance
-    """
-    codec = AudioCodec()
-    
-    for child in root.getchildren():
-        if child.tag == "name":
-            codec.name = child.text.strip()
-        elif child.tag == "container":
-            codec.container = child.text.strip()
-        elif child.tag == "width":
-            codec.width = _parse_range(child.text.strip())
-        elif child.tag == "depth":
-            codec.depth = _parse_range(child.text.strip())
-        elif child.tag == "channels":
-            codec.channels = _parse_range(child.text.strip())
-        elif child.tag == "rate":
-            codec.rate = _parse_range(child.text.strip())
-        elif child.tag == "passes":
-            for command in child.getchildren():
-                codec.passes.append(command.text.strip())
-    
-    return codec
-
-def _load_video_codec(root):
-    """
-        Load a video codec from a given xml element.
-        
-        @type root: xml.etree.Element
-        @param root: An video codec element
-        @rtype: VideoCodec
-        @return: A new VideoCodec instance
-    """
-    codec = VideoCodec()
-    
-    for child in root.getchildren():
-        if child.tag == "name":
-            codec.name = child.text.strip()
-        elif child.tag == "container":
-            codec.container = child.text.strip()
-        elif child.tag == "width":
-            codec.width = _parse_range(child.text.strip())
-        elif child.tag == "height":
-            codec.height = _parse_range(child.text.strip())
-        elif child.tag == "rate":
-            codec.rate = _parse_range(child.text.strip(), Fraction)
-        elif child.tag == "passes":
-            for command in child.getchildren():
-                codec.passes.append(command.text.strip())
-    
-    return codec
-
-def _load_preset(root):
-    """
-        Load a preset from a given xml element.
-        
-        @type root: xml.etree.Element
-        @param root: An preset element
-        @rtype: Preset
-        @return: A new preset instance
-    """
-    preset = Preset()
-    
-    for child in root.getchildren():
-        if child.tag == "name":
-            preset.name = child.text.strip()
-        elif child.tag == "container":
-            preset.container = child.text.strip()
-        elif child.tag == "extension":
-            preset.extension = child.text.strip()
-        elif child.tag == "audio":
-            preset.acodec = _load_audio_codec(child)
-        elif child.tag == "video":
-            preset.vcodec = _load_video_codec(child)
-    
-    if preset.acodec and not preset.acodec.container:
-        preset.acodec.container = preset.container
-    
-    if preset.vcodec and not preset.vcodec.container:
-        preset.vcodec.container = preset.container
-        
-    return preset
+    def __init__(self, name=None, container=None, rate=None, passes=None, width=None, height=None):
+        Codec.__init__(self, name=name, container=container, passes=passes)
+        self.rate = rate and rate or (Fraction("1"), Fraction("60"))
+        self.width = width and width or (2, 1920)
+        self.height = height and height or (2, 1080)
 
 def load(filename):
     """
@@ -440,34 +441,13 @@ def load(filename):
         @rtype: Device
         @return: A new device instance loaded from the file
     """
-    tree = xml.etree.ElementTree.parse(filename)
-    
-    device = Device()
+    device = Device.from_json(open(filename).read())
     
     device.filename = filename
     
-    for child in tree.getroot().getchildren():
-        if child.tag == "make":
-            device.make = child.text.strip()
-        elif child.tag == "model":
-            device.model = child.text.strip()
-        elif child.tag == "description":
-            device.description = child.text.strip()
-        elif child.tag == "author":
-            device.author = _load_author(child)
-        elif child.tag == "version":
-            device.version = child.text.strip()
-        elif child.tag == "preset":
-            preset = (_load_preset(child))
-            device.presets[preset.name] = preset
-        elif child.tag == "icon":
-            device.icon = child.text.strip()
-        elif child.tag == "default":
-            device.default = child.text.strip()
-    
     _log.debug(_("Loaded device %(device)s (%(presets)d presets)") % {
         "device": device.name,
-        "presets": len(device.presets)
+        "presets": len(device.presets),
     })
     
     return device
@@ -483,8 +463,11 @@ def load_directory(directory):
     """
     global _presets
     for filename in os.listdir(directory):
-        if filename.endswith("xml"):
-            _presets[filename[:-4]] = load(os.path.join(directory, filename))
+        if filename.endswith("json"):
+            try:
+                _presets[filename[:-5]] = load(os.path.join(directory, filename))
+            except:
+                _log.warning("Problem loading %s!" % filename)
     return _presets
 
 def get():
@@ -512,7 +495,29 @@ def version_info():
         
     return info
 
-def install_preset(location, name):
+def extract(stream):
+    """
+        Extract a preset file into the user's local presets directory.
+        
+        @type stream: a file-like object
+        @param stream: The opened bzip2-compressed tar file of the preset
+        @rtype: list
+        @return: The installed device preset shortnames ["name1", "name2", ...]
+    """
+    local_path = os.path.expanduser(os.path.join("~", ".arista", "presets"))
+    
+    if not os.path.exists(local_path):
+        os.makedirs(local_path)
+        
+    tar = tarfile.open(mode="r|bz2", fileobj=stream)
+    _log.debug(_("Extracting %(filename)s") % {
+        "filename": hasattr(stream, "name") and stream.name or "data stream",
+    })
+    tar.extractall(path=local_path)
+    
+    return [x[:-5] for x in tar.getnames() if x.endswith(".json")]
+
+def fetch(location, name):
     """
         Attempt to fetch and install a preset. Presets are always installed
         to ~/.arista/presets/.
@@ -521,41 +526,37 @@ def install_preset(location, name):
         @param location: The location of the preset
         @type name: str
         @param name: The name of the preset to fetch, without any extension
+        @rtype: list
+        @return: The installed device preset shortnames ["name1", "name2", ...]
     """
-    local_path = os.path.expanduser(os.path.join("~", ".arista", "presets"))
-    
-    if not os.path.exists(local_path):
-        os.makedirs(local_path)
-    
     if not location.endswith("/"):
         location = location + "/"
     
-    for ext in ["xml", "svg"]:
-        path = ".".join([location + name, ext])
-        _log.debug(_("Fetching %(location)s") % {
+    path = location + name + ".tar.bz2"
+    _log.debug(_("Fetching %(location)s") % {
+        "location": path,
+    })
+    
+    updated = []
+    
+    try:
+        f = urllib2.urlopen(path)
+        updated += extract(f)
+    except Exception, e:
+        _log.warning(_("There was an error fetching and installing " \
+                       "%(location)s: %(error)s") % {
             "location": path,
+            "error": str(e),
         })
-        
-        try:
-            f = urllib2.urlopen(path)
-            local_file = os.path.join(local_path, ".".join([name, ext]))
-            _log.debug(_("Writing to %(file)s") % {
-                "file": local_file,
-            })
-            open(local_file, "w").write(f.read())
-        except Exception, e:
-            _log.error(_("There was an error fetching and installing " \
-                         "%(location)s: %(error)s") % {
-                "location": path,
-                "error": str(e),
-            })
+    
+    return updated
 
 def check_for_updates(location = UPDATE_LOCATION):
     """
         Check for updated presets from a central server.
         
         @type location: str
-        @param location: The directory where presets.txt and all preset files
+        @param location: The directory where latest.txt and all preset files
                          can be found on the server
         @rtype: list
         @return: A list of [(location, name), (location, name), ...] for each
@@ -569,7 +570,7 @@ def check_for_updates(location = UPDATE_LOCATION):
         location = location + "/"
     
     try:
-        f = urllib2.urlopen(location + "presets.txt")
+        f = urllib2.urlopen(location + "latest.txt")
     except urllib2.URLError:
         return updates
     
@@ -600,25 +601,12 @@ def check_for_updates(location = UPDATE_LOCATION):
                                 "location": location,
                                 "error": str(e),
                             })
-                else:
-                    _log.debug(_("Found new device preset %(name)s") % {
-                        "name": name,
-                    })
-                    try:
-                        updates.append((location, name))
-                    except Exception, e:
-                        _log.error(_("Error installing preset %(name)s " \
-                                     "from %(location)s: %(error)s") % {
-                            "name": name,
-                            "location": location,
-                            "error": str(e),
-                        })
             else:
                 _log.warning(_("Malformed plugin version line %(line)s") % {
                     "line": line,
                 })
     except:
-        _log.warning(_("There was a problem accessing %(location)spresets.txt!") % {
+        _log.warning(_("There was a problem accessing %(location)slatest.txt!") % {
             "location": location,
         })
     
@@ -638,13 +626,20 @@ def check_and_install_updates(location = UPDATE_LOCATION):
     
     if updates:
         for loc, name in updates:
-            install_preset(loc, name)
+            fetch(loc, name)
     else:
         _log.debug(_("All device presets are up to date!"))
 
-# Automatically load presets - system, home, current path
-for path in reversed(utils.get_search_paths()):
-    full = os.path.join(path, "presets")
-    if os.path.exists(full):
-        load_directory(full)
+def reset():
+    # Automatically load presets - system, home, current path
+    global _presets
+    
+    _presets = {}
+    
+    for path in reversed(utils.get_search_paths()):
+        full = os.path.join(path, "presets")
+        if os.path.exists(full):
+            load_directory(full)
+
+reset()
 
