@@ -7,7 +7,7 @@
     
     License
     -------
-    Copyright 2009 Daniel G. Taylor <dan@programmer-art.org>
+    Copyright 2009 - 2010 Daniel G. Taylor <dan@programmer-art.org>
     
     This file is part of Arista.
 
@@ -33,6 +33,17 @@ import os.path
 import sys
 import time
 
+# Default to 2 CPUs as most seem to be dual-core these days
+CPU_COUNT = 2
+try:
+    import multiprocessing
+    try:
+        CPU_COUNT = multiprocessing.cpu_count()
+    except NotImplementedError:
+        pass
+except ImportError:
+    pass
+
 import gobject
 import gst
 
@@ -69,15 +80,20 @@ class PipelineException(TranscoderException):
 # Transcoder Options
 # =============================================================================
 
-class InputOptions(object):
+class TranscoderOptions(object):
     """
-        Options pertaining to the input location, subtitles, etc.
+        Options pertaining to the input/output location, presets, 
+        subtitles, etc.
     """
-    def __init__(self, uri, subfile = None, font = "Sans Bold 16",
-                 deinterlace = None):
+    def __init__(self, uri = None, preset = None, output_uri = None, 
+                 subfile = None, font = "Sans Bold 16", deinterlace = None):
         """
             @type uri: str
             @param uri: The URI to the input file, device, or stream
+            @type preset: Preset
+            @param preset: The preset to convert to
+            @type output_uri: str
+            @param output_uri: The URI to the output file, device, or stream
             @type subfile: str
             @param subfile: The location of the subtitle file
             @type font: str
@@ -85,14 +101,16 @@ class InputOptions(object):
             @type deinterlace: bool
             @param deinterlace: Force deinterlacing of the input data
         """
-        self.reset(uri, subfile, font, deinterlace)
+        self.reset(uri, preset, output_uri, subfile, font, deinterlace)
     
-    def reset(self, uri = None, subfile = None, font = "Sans Bold 16",
-              deinterlace = None):
+    def reset(self, uri = None, preset = None, output_uri = None,
+              subfile = None, font = "Sans Bold 16", deinterlace = None):
         """
             Reset the input options to nothing.
         """
         self.uri = uri
+        self.preset = preset
+        self.output_uri = output_uri
         self.subfile = subfile
         self.font = font
         self.deinterlace = deinterlace
@@ -103,7 +121,7 @@ class InputOptions(object):
 
 class Transcoder(gobject.GObject):
     """
-    
+        The transcoder - converts media between formats.
     """
     __gsignals__ = {
         "discovered": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
@@ -115,49 +133,119 @@ class Transcoder(gobject.GObject):
                    (gobject.TYPE_PYOBJECT,         # bus
                     gobject.TYPE_PYOBJECT)),       # message
         "complete": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, tuple()),
+        "error": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
+                 (gobject.TYPE_PYOBJECT,)),        # error
     }
     
-    def __init__(self, input_options, outfile, preset):
+    def __init__(self, options):
         """
-            @type infile: InputOptions
-            @param infile: The input options to process (uri, subtitles, etc)
-            @type outfile: str
-            @param outfile: The output path to save to
-            @type preset: Preset
-            @param preset: The preset instance to use for the conversion
+            @type options: TranscoderOptions
+            @param options: The options, like input uri, subtitles, preset, 
+                            output uri, etc.
         """
         self.__gobject_init__()
-        self.input_options = input_options
-        self.outfile = outfile
-        self.preset = preset
+        self.options = options
         
         self.pipe = None
         
         self.enc_pass = 0
         
-        def _got_info(info, is_media):
-            self.info = info
-            self.emit("discovered", info, is_media)
-            
-            if info.is_video or info.is_audio:
-                self._setup_pass()
-                self.start()
+        self._percent_cached = 0
+        self._percent_cached_time = 0
         
-        self.info = None
-        self.discoverer = discoverer.Discoverer(input_options.uri)
-        self.discoverer.connect("discovered", _got_info)
-        self.discoverer.discover()
+        if options.uri.startswith("dvd://") and len(options.uri.split("@")) < 2:
+            # This is a DVD and no title is yet selected... find the best
+            # candidate by searching for the longest title!
+            self.options.uri += "@1"
+            self.dvd_infos = []
+            
+            def _got_info(info, is_media):
+                self.dvd_infos.append([discoverer, info])
+                parts = self.options.uri.split("@")
+                fname = parts[0]
+                title = int(parts[1])
+                if title >= 8:
+                    # We've checked 8 titles, let's give up and pick the
+                    # most likely to be the main feature.
+                    longest = 0
+                    self.info = None
+                    for disco, info in self.dvd_infos:
+                        if info.length > longest:
+                            self.discoverer = disco
+                            self.info = info
+                            longest = info.length
+                    
+                    if not self.info:
+                        self.emit("error", _("No valid DVD title found!"))
+                        return
+                    
+                    self.options.uri = self.info.filename
+                    
+                    _log.debug(_("Longest title found is %(filename)s") % {
+                        "filename": self.options.uri,
+                    })
+                    
+                    self.emit("discovered", self.info, self.info.is_video or self.info.is_audio)
+                    
+                    if self.info.is_video or self.info.is_audio:
+                        try:
+                            self._setup_pass()
+                        except PipelineException, e:
+                            self.emit("error", str(e))
+                            return
+                        
+                        self.start()
+                        return
+                
+                self.options.uri = fname + "@" + str(title + 1)
+                self.discoverer = discoverer.Discoverer(options.uri)
+                self.discoverer.connect("discovered", _got_info)
+                self.discoverer.discover()
+            
+            self.discoverer = discoverer.Discoverer(options.uri)
+            self.discoverer.connect("discovered", _got_info)
+            self.discoverer.discover()
+        
+        else:
+            def _got_info(info, is_media):
+                self.info = info
+                self.emit("discovered", info, is_media)
+                
+                if info.is_video or info.is_audio:
+                    try:
+                        self._setup_pass()
+                    except PipelineException, e:
+                        self.emit("error", str(e))
+                        return
+                        
+                    self.start()
+            
+            self.info = None
+            self.discoverer = discoverer.Discoverer(options.uri)
+            self.discoverer.connect("discovered", _got_info)
+            self.discoverer.discover()
     
     @property
     def infile(self):
         """
             Provide access to the input uri for backwards compatibility after
-            moving to InputOptions for uri, subtitles, etc.
+            moving to TranscoderOptions for uri, subtitles, etc.
             
             @rtype: str
             @return: The input uri to process
         """
-        return self.input_options.uri
+        return self.options.uri
+    
+    @property
+    def preset(self):
+        """
+            Provide access to the output preset for backwards compatibility
+            after moving to TranscoderOptions.
+            
+            @rtype: Preset
+            @return: The output preset
+        """
+        return self.options.preset
     
     def _get_source(self):
         """
@@ -169,22 +257,28 @@ class Transcoder(gobject.GObject):
             @return: Source to prepend to gst-launch style strings.
         """
         if self.infile.startswith("dvd://"):
-            if self.input_options.deinterlace is None:
-                self.input_options.deinterlace = True
-            parts = self.infile[6:].split("@")
+            parts = self.infile.split("@")
+            device = parts[0][6:]
+            
+            title = 1
             if len(parts) > 1:
-                title = parts[1]
-                src = "dvdreadsrc device=\"%s\" title=\"%s\"" % (parts[0], parts[1])
-            else:
-                src = "dvdreadsrc device=\"%s\"" % parts[0]
-        elif self.infile.startswith("v4l://"):
-            src = "v4lsrc device=\"%s\"" % self.infile[6:]
-        elif self.infile.startswith("v4l2://"):
-            src = "v4l2src device=\"%s\"" % self.infile[7:]
+                try:
+                    title = int(parts[1])
+                except:
+                    title = 1
+            
+            if self.options.deinterlace is None:
+                self.options.deinterlace = True
+            
+            return "dvdreadsrc device=\"%s\" title=%d ! decodebin2 name=dmux" % (device, title)
+        elif self.infile.startswith("v4l://") or self.infile.startswith("v4l2://"):
+            filename = self.infile
+        elif self.infile.startswith("file://"):
+            filename = self.infile
         else:
-            src = "filesrc location=\"%s\"" % self.infile
-        
-        return src + " name=source"
+            filename = "file://" + self.infile
+            
+        return "uridecodebin uri=\"%s\" name=dmux" % filename
     
     def _setup_pass(self):
         """
@@ -230,8 +324,8 @@ class Transcoder(gobject.GObject):
         
         src = self._get_source()
         
-        cmd = "%s ! decodebin2 name=dmux %s filesink name=sink " \
-              "location=\"%s\"" % (src, mux_str, self.outfile)
+        cmd = "%s %s filesink name=sink " \
+              "location=\"%s\"" % (src, mux_str, self.options.output_uri)
             
         if self.info.is_video and self.preset.vcodec:
             # =================================================================
@@ -290,8 +384,12 @@ class Transcoder(gobject.GObject):
                 px = (hmin - height) / 2
                 vbox = "videobox top=%i bottom=%i ! " % (-px, -px)
             
-            if self.info.videocaps[0].has_key("pixel-aspect-ratio"):
-                width = int(width * float(self.info.videocaps[0]["pixel-aspect-ratio"]))
+            try:
+                if self.info.videocaps[0].has_key("pixel-aspect-ratio"):
+                    width = int(width * float(self.info.videocaps[0]["pixel-aspect-ratio"]))
+            except KeyError:
+                # The videocaps we are looking for may not even exist, just ignore
+                pass
             
             # FIXME Odd widths / heights seem to freeze gstreamer
             if width % 2:
@@ -353,26 +451,33 @@ class Transcoder(gobject.GObject):
             # Setup the video encoder and options
             # =================================================================
             vencoder = "%s %s" % (self.preset.vcodec.name,
-                                  self.preset.vcodec.passes[self.enc_pass])
+                                  self.preset.vcodec.passes[self.enc_pass] % {
+                                    "threads": CPU_COUNT,
+                                  })
             
             deint = ""
-            if self.input_options.deinterlace:
+            if self.options.deinterlace:
                 deint = " ffdeinterlace ! "
             
             sub = ""
-            if self.input_options.subfile:
+            if self.options.subfile:
                 # Render subtitles onto the video stream
                 sub = "textoverlay font-desc=\"%(font)s\" name=txt ! " % {
-                    "font": self.input_options.font,
+                    "font": self.options.font,
                 }
                 cmd += " filesrc location=\"%(subfile)s\" ! subparse ! txt." % {
-                    "subfile": self.input_options.subfile
+                    "subfile": self.options.subfile
                 }
             
+            vmux = premux
+            if container in ["qtmux", "webmmux", "ffmux_dvd", "matroskamux"]:
+                if premux.startswith("mux"):
+                    vmux += "video_%d"
+            
             cmd += " dmux. ! queue ! ffmpegcolorspace ! videorate !" \
                    "%s %s videoscale ! %s ! %s%s ! tee " \
-                   "name=videotee ! queue ! %svideo_00" % \
-                   (deint, sub, self.vcaps.to_string(), vbox, vencoder, premux)
+                   "name=videotee ! queue ! %s" % \
+                   (deint, sub, self.vcaps.to_string(), vbox, vencoder, vmux)
             
         if self.info.is_audio and self.preset.acodec and \
            self.enc_pass == len(self.preset.vcodec.passes) - 1:
@@ -382,24 +487,31 @@ class Transcoder(gobject.GObject):
             element = gst.element_factory_make(self.preset.acodec.name,
                                                "aencoder")
             
-            # TODO: Add rate limits based on encoder sink below
+            fields = {}
             for cap in element.get_pad("sink").get_caps():
-                for field in ["width", "depth", "channels"]:
+                for field in ["width", "depth", "rate", "channels"]:
                     if cap.has_field(field):
+                        if field not in fields:
+                            fields[field] = [0, 0]
                         value = cap[field]
                         if isinstance(value, gst.IntRange):
                             vmin, vmax = value.low, value.high
                         else:
                             vmin, vmax = value, value
                         
-                        cur = getattr(self.preset.acodec, field)
-                        if cur[0] < vmin:
-                            cur = (vmin, cur[1])
-                            setattr(self.preset.acodec, field, cur)
-                    
-                        if cur[1] > vmax:
-                            cur = (cur[0], vmax)
-                            setattr(self.preset.acodec, field, cur)
+                        if vmin < fields[field][0]:
+                            fields[field][0] = vmin
+                        if vmax > fields[field][1]:
+                            fields[field][1] = vmax
+            
+            for name, (amin, amax) in fields.items():
+                cur = getattr(self.preset.acodec, field)
+                if cur[0] < amin:
+                    cur = (amin, cur[1])
+                    setattr(self.preset.acodec, field, cur)
+                if cur[1] > amax:
+                    cur = (cur[0], amax)
+                    setattr(self.preset.acodec, field, cur)
             
             # =================================================================
             # Prepare audio capabilities
@@ -408,16 +520,12 @@ class Transcoder(gobject.GObject):
                 current = getattr(self.info, "audio" + attribute)
                 amin, amax = getattr(self.preset.acodec, attribute)
                 
-                aminvalue = amin
-                amaxvalue = amax
-                
-                if current > amaxvalue:
-                    for acap in self.acaps:
-                        acap[attribute] = amax
-                elif current < aminvalue:
-                    for acap in self.acaps:
+                for acap in self.acaps:
+                    if amin < amax:
+                        acap[attribute] = gst.IntRange(amin, amax)
+                    else:
                         acap[attribute] = amin
-
+            
             # =================================================================
             # Add audio transcoding pipeline to command
             # =================================================================
@@ -425,11 +533,18 @@ class Transcoder(gobject.GObject):
                        self.preset.acodec.passes[ \
                             len(self.preset.vcodec.passes) - \
                             self.enc_pass - 1 \
-                       ]
-                
+                       ] % {
+                            "threads": CPU_COUNT,
+                       }
+            
+            amux = premux
+            if container in ["qtmux", "webmmux", "ffmux_dvd", "matroskamux"]:
+                if premux.startswith("mux"):
+                    amux += "audio_%d"
+            
             cmd += " dmux. ! queue ! audioconvert ! audiorate ! " \
-                   "audioresample ! %s ! %s ! %saudio_00" % \
-                   (self.acaps.to_string(), aencoder, premux)
+                   "audioresample ! %s ! %s ! %s" % \
+                   (self.acaps.to_string(), aencoder, amux)
         
         # =====================================================================
         # Build the pipeline and get ready!
@@ -543,7 +658,7 @@ class Transcoder(gobject.GObject):
         """
         duration = max(self.info.videolength, self.info.audiolength)
         
-        if not duration:
+        if not duration or duration < 0:
             return 0.0, _("Unknown")
         
         try:
@@ -554,9 +669,16 @@ class Transcoder(gobject.GObject):
             raise TranscoderStatusException(_("No pipeline to query!"))
         
         percent = pos / float(duration)
-        if percent == 0:
+        if percent <= 0.0:
             return 0.0, _("Unknown")
         
+        if self._percent_cached == percent and time.time() - self._percent_cached_time > 5:
+            self.pipe.post_message(gst.message_new_eos(self.pipe))
+        
+        if self._percent_cached != percent:
+            self._percent_cached = percent
+            self._percent_cached_time = time.time()
+        
         total = 1.0 / percent * (time.time() - self.start_time)
         rem = total - (time.time() - self.start_time)
         min = rem / 60
